'''
Created on May 7, 2019

@author: reynolds
'''

import sys, os, struct

from wrtdk.parser.parser import parser
from wrtdk.parser.msg.wrtmsg import udp_wrapper
from wrtdk.io.sim.sim_simulator import sim_simulator

class gprip(parser):
    '''
    classdocs
    '''

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()
        self.reset()
        self.id = b'$GPRIP'
        
    def reset(self):
        self.depth = self._nan()
        self.alt = self._nan()
        self.lat = self._nan()
        self.lon = self._nan()
        self.lat_fix = self._nan()
        self.lon_fix = self._nan()
        self.fix = self._nan()
        self.fix_time = self._nan()
        self.time_ms = self._nan()
        self.state = self._nan()
        self.roll = self._nan()
        self.pitch = self._nan()
        self.yaw = self._nan()
        self.speed = self._nan()
    
    def parse(self,msg):
        self.reset()
        try:
            _,self.time_ms,self.state,self.fix,self.fix_time,self.lat_fix,self.lon_fix,self.lat,self.lon,self.depth,self.alt,self.roll,self.pitch,self.yaw,self.speed = struct.unpack('>6sdBH11d',msg)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def get(self,time=0,state=0,
            fix=0,time_fix=0,lat_fix=0,lon_fix=0,
            lat=0,lon=0,
            depth=0,alt=0,speed_mps=0,
            roll=0,pitch=0,yaw=0):
        try:
            return struct.pack('>6sdBH11d',
                           self.id,
                           time,state,
                           fix,time_fix,lat_fix,lon_fix,
                           lat,lon,
                           depth,alt,
                           roll,pitch,yaw,
                           speed_mps)
        except Exception as e:
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
            return None
    
    def getData(self):
        ''' returns the data form the gprip message in a list
        0) epoch time in ms
        1) state
        2) the navigation fix
        3) the fix epoch time in ms
        4) the fix latitude in dd
        5) the fix longtude in dd
        6) the estimated latitude in dd
        7) the estimated longitude in dd
        8) the depth in m
        9) the altitude above the sea floor in m
        10) the roll in radians
        11) the pitch in radians
        12) the yaw in radians
        13) the vehicle speed in mps
        '''
        return [self.time_ms,self.state,
                self.fix,self.fix_time,self.lat_fix,self.lon_fix,
                self.lat,self.lon,
                self.depth,self.alt,
                self.roll,self.pitch,self.yaw,
                self.speed]

def test_gprip():
    m = gprip()
    f = r'C:\Users\reynolds\Documents\1704_madunit\data\20190507_gprip\GPRIP_SWXXX.dat'
    msg = get_msg(f,20)
    for mm in msg:
        print('len: %d msg:%s' % (len(mm),mm))
        m.parse(mm)
        print(m.getData())
    
def get_msg(filename,n=1):
    reader = sim_simulator()
    reader.read(filename,'$GPRIP')
    payload = []
    wrapper = udp_wrapper()
    while len(payload) < n:
        msg = reader.getNext()
        wrapper.parse(msg[0:wrapper.LENGTH])
        if not wrapper.hasErrored():
            payload.append(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
    
    return payload

if __name__ == '__main__':
    test_gprip()


'''
Created on Nov 13, 2019

@author: reynolds
'''
import unittest, sys
from wrtdk.io.streamer.em_streamer import em_streamer
from wrtdk.comms.comms import tcp_port
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QMainWindow, QApplication

class main_window(QMainWindow):
    
    def __init__(self):
        super().__init__()
        
        s = tcp_port('169.254.10.100',10011,1)
        self.t = em_streamer(port=s)
        self.t.new_timeout.connect(self._new_timeout)
        self.t.new_em.connect(self._new_em)
        self.t.new_conf.connect(self._new_conf)
        self.t.new_gga.connect(self._new_gga)
        self.t.start()
        
        self.recv = [False] * 4
        
    def check(self):
        check = True
        for r in self.recv[1::]: check = check and r
        if check:
            self.t.stop()
            self.t.wait()
            self.close()
        
    @pyqtSlot()
    def _new_timeout(self):
        print('timeout')
        self.recv[0] = True
        self.check()
    
    @pyqtSlot(int,int,list)
    def _new_em(self,mtype,mlen,data):
        print(mtype,mlen,data)
        self.recv[1] = True
        self.check()
    
    @pyqtSlot(int,int,list)
    def _new_conf(self,mtype,mlen,data):
        print(mtype,mlen,data)
        self.recv[2] = True
        self.check()
    
    @pyqtSlot(int,int,list)
    def _new_gga(self,mtype,mlen,data):
        print(mtype,mlen,data)
        self.recv[3] = True
        self.check()

class Test(unittest.TestCase):
    
    def setUp(self):
        app = QApplication(sys.argv)
        self.widget = main_window()
        self.widget.show()
        app.exec_()

    def test_thread_running(self):
        self.assertTrue(self.widget.t.isRunning())
        
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Oct 31, 2019

@author: reynolds
'''

import os, sys

from wrtdk.io.streamer.streamer import LoggingStreamer
from PyQt5.QtCore import pyqtSignal
from wrtdk.data.buffer.buffer import byte_buffer
from wrtdk.parser.nmea.nmea import gpgga
from wrtdk.parser.em.em3d.em.em import emv1,emv2
from wrtdk.parser.em.em3d.conf.conf import confv3,confv5

class em_streamer(LoggingStreamer):
    '''
    classdocs
    '''
    
    EM_SOL = 116
    GGA_SOL = 36
    CONF_SOL = 67
    EOL = bytes([10])
    SOL = bytes([EM_SOL,CONF_SOL,GGA_SOL])
    EM = 0
    CONF = 1
    GGA = 2
    RAW = 3
    
    new_timeout = pyqtSignal()# timeout signal
    new_em = pyqtSignal(int,int,list)#type,list length,data
    new_conf = pyqtSignal(int,int,list)#type, list length,data
    new_gga = pyqtSignal(int,int,list)#type, list length, data
    new_raw = pyqtSignal(int,int,bytes)#type, list length, data

    def __init__(self,port=None,debug=False,filename=None,dt=0.1,version=1):
        '''
        Constructor
        '''
        super().__init__(debug=debug,port=port,dt=dt)
        
        self._buffer = byte_buffer()
        
        # define parsers
        self.gga = gpgga()
        self._version(version)
        if self._debug and os.path.join(filename):
            print('Simulation is currently not functional - rwr 10/31/19')
            self.filename=filename
        else:self.filename=None
        
    def _version(self,version):
        ''' sets the version of the parser to be used '''
        if version == 2:# second version
            self.em = emv2()
            self.conf - confv5()
        else:# assume first version
            self.em = emv1()
            self.conf = confv3()
            
    def run(self):
        self.set_mode()
        
        i = 0
        
        while self._running:
            try:
                msg,_ = self.port.read(2000)
            except Exception as e:
                exc_type, _, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
                self.new_timeout.emit()
                continue
            
            if msg is None: return
            
            i += 1
            
            if self.isLogging(): self.writer.write(msg)
            
            self._buffer.append(msg)
            
            m = self.parse()
            
            while m is not None:
                self.send(m)
                m = self.parse()
                
            
        self.port.close()           
            
    def parse(self):
        if self._buffer.is_empty(): return
        
        # find a start byte
        try:
            b = bytes([self._buffer.get()])
        except Exception as e:
            print('error',str(e),self._buffer)
            return
        while not self.is_sol(b) and not self._buffer.is_empty(): 
            b = bytes([self._buffer.get()])
        
        buff = bytes()
        
        while not self._buffer.is_empty():
            buff = buff + b
            
            # check if the message is full
            if self.is_eol(b) and self.is_sol(self._buffer.view()):
                self._buffer.remove_head()
                return buff
            
            # get the next byte
            b = bytes([self._buffer.get()])
        
        self._buffer.reset_position()
        
        if self._buffer.get_length() > 2000: self._buffer.clear()
            
        return None
    
    def send(self,b):
        self.new_raw.emit(self.RAW,len(b),b)
        
        if self.is_em(b[0]):
            if not self.em.is_set(): return
            self.em.parse(b)
            if not self.em.hasErrored():
                data = self.em.getData()
                self.new_em.emit(self.EM,
                                 len(data),
                                 data)
        elif self.is_conf(b[0]):
            self.conf.parse(b)
            if not self.conf.hasErrored():
                if not self.em.is_set():
                    self.em.set_bins(self.conf.get_bins())
                data = self.conf.getData()
                self.new_conf.emit(self.CONF,
                                   len(data),
                                   data)
        elif self.is_gga(b[0]):
            self.gga.parse(b)
            if not self.gga.hasErrored():
                data = self.gga.getData()
                self.new_gga.emit(self.GGA,
                                  len(data),
                                  data)
    
    def is_conf(self,b):
        return b == self.SOL[1]
    
    def is_em(self,b):
        return b == self.SOL[0]
    
    def is_gga(self,b):
        return b == self.SOL[2]
        
    def is_sol(self,b):
        return b in self.SOL
    
    def is_eol(self,b):
        return b in self.EOL


'''
Created on Oct 16, 2018

@author: reynolds
'''

import visa

class instrument(object):
    '''
    classdocs
    '''


    def __init__(self, params):
        '''
        Constructor
        '''
        self._params = params.copy()
        self.oppar(self._params)
        
        self._rm = visa.ResourceManager('@py')
        
    def oppar(self,par):
        pass
    
    def measure(self):
        pass
    
    def write(self,fn=''):
        pass
    
    def isOpen(self):
        return False
    
    


'''
Created on Dec 26, 2019

@author: Reynolds
'''
import unittest
from wrtdk.parser.alt.lidar_lite_v3 import lidar_lite_v3


class Test(unittest.TestCase):

    def setUp(self):
        self.msg = b'\x9a\x9a\x00S'
        self.lidar = lidar_lite_v3()

    def test_lidar_lite_v3(self):
        self.lidar.parse(self.msg)
        self.assertFalse(self.lidar.hasErrored())
        d1 = self.lidar.getData()
        
        msg = self.lidar.get(health=d1[0],
                             reg=d1[1],
                             roverflow=d1[3],
                             soverflow=d1[4],
                             dpeak=d1[2],
                             busy=d1[5],
                             strength=d1[6],
                             d=d1[-1])
        
        self.lidar.parse(msg)
        self.assertFalse(self.lidar.hasErrored())
        
        d2 = self.lidar.getData()
        
        self.assertEqual(d1,d2)


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Nov 13, 2019

@author: reynolds
'''
import unittest

from wrtdk.comms.comms import tcp_port

ADDR = 'localhost'
PORT = 10011

class Test(unittest.TestCase):
    
    def test(self):
        s = tcp_port(connect=False)
        
        _open = s.open(ADDR,PORT,1)
        self.assertTrue(_open)
        
        msg,addr = s.read(1000)
        print(addr,msg)
        self.assertNotEqual(msg,None)
        
        _close = s.close()
        self.assertTrue(_close)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_open']
    unittest.main()


'''
Created on Feb 14, 2019

@author: reynolds
'''
import sys, os

from PyQt5.QtWidgets import QApplication,QMainWindow, QPushButton, QGridLayout,QWidget
from PyQt5.QtCore import pyqtSlot
from wrtdk.io.streamer.simstreamer import simstreamer
from wrtdk.comms.comms import UDPServerPort

class test(QMainWindow):
    '''
    classdocs
    '''

    def __init__(self):
        ''' Constructor '''
        QMainWindow.__init__(self)
        
        main = QWidget(self)
        btn = QPushButton('Connect')
        btn.clicked.connect(self._connect)
        log = QPushButton('Log')
        log.clicked.connect(self._log)
        
        l = QGridLayout()
        l.addWidget(btn)
        l.addWidget(log)
        
        main.setLayout(l)
        self.setFocus()
        self.setCentralWidget(main)
        
        self._port = None
        self._thread = None
        
    def _connect(self):
        if self._port is None:
            f = r'C:\Users\reynolds\Documents\1704_madunit\data\20190507_gprip\GPRIP_SWXXX.dat'
            f = r'C:\Users\reynolds\Documents\1607-te_multi-static\data\20190506_gantry\test_1a.dat'
            print(os.path.exists(f))
            self._port = UDPServerPort('',7654,1)
            self._thread = simstreamer(self._port,
                                       debug=True,
                                       filename=f,
                                       dt=.001)
            self._thread.new_mcu.connect(self._new_mcu)
            self._thread.new_gga.connect(self._new_gga)
            self._thread.new_rmc.connect(self._new_rmc)
            self._thread.new_hdt.connect(self._new_hdt)
            self._thread.new_rmc.connect(self._new_rmc)
            self._thread.new_vhw.connect(self._new_vhw)
            self._thread.new_vtg.connect(self._new_vtg)
            self._thread.new_gprip.connect(self._new_gprip)
            self._thread.new_qmag.connect(self._new_qmag)
            self._thread.new_mfam.connect(self._new_mfam)
            self._thread.new_vmag.connect(self._new_vmag)
            self._thread.new_imu.connect(self._new_imu)
            self._thread.new_sonar.connect(self._new_sonar)
            self._thread.new_lidar.connect(self._new_lidar)
            self._thread.new_sys.connect(self._new_sys)
            self._thread.new_timeout.connect(self._new_timeout)
            self._thread.new_rtcom_mag.connect(self._new_com_mag)
            self._thread.new_rtdet_mag.connect(self._new_det_mag)
            self._thread.new_rtinv_mag.connect(self._new_inv_mag)
            self._thread.new_rttrk_mag.connect(self._new_trk_mag)
            self._thread.start()
        else:
            self._thread.stop()
            self._thread.wait()
            self._thread = None
            self._port = None
    
    def _log(self):
        if self._thread.isRunning():
            if self._thread.isLogging():
                self._thread.stopLog()
            else:
                self._thread.startLog('test.dat','wb')
            
        
    @pyqtSlot(list,str,int,int)
    def _new_qmag(self,data,status,system,sensor):
        '''  updates the total field  '''
        print('TF:',data[0],'strength:',data[1],'state:',status,system,sensor)
    
    @pyqtSlot(list,str,int,int)
    def _new_sys(self,data,status,system,sensor):
        ''' updates the system state '''
        print(data,status,system,sensor)
            
    @pyqtSlot(list,str,int,int)
    def _new_gga(self,data,status,system,sensor):
        '''  updates the gga display  '''
        print(data,status,system,sensor)
    
    @pyqtSlot(list,str,int,int)
    def _new_rmc(self,data,status,system,sensor):
        ''' updates the gprmc state '''
        print(data,status,system,sensor)
            
    @pyqtSlot(list,str,int,int)
    def _new_vtg(self,data,status,system,sensor):
        ''' updates the gpvtg state '''
        print(data,status,system,sensor)
            
    @pyqtSlot(list,str,int,int)
    def _new_vhw(self,data,status,system,sensor):
        ''' updates the gpvhw state '''
        print(data,status,system,sensor)
            
    @pyqtSlot(list,str,int,int)
    def _new_hdt(self,data,status,system,sensor):
        ''' updates the gphdt state '''
        print(data,status,system,sensor)
        
    @pyqtSlot(list,str,int,int)
    def _new_gprip(self,data,status,system,sensor):
        print('GPRIP',data,status,system,sensor)
    
    @pyqtSlot(list,str,int,int)
    def _new_mfam(self,data,status,system,sensor):
        ''' updates the MFAM tf plot and statuses '''
        print('MFAM',data,status,system,sensor)
            
    @pyqtSlot(list,str,int,int)
    def _new_imu(self,data,status,system,sensor):
        ''' updates the IMU readout '''
        print(data,status,system,sensor)
            
    @pyqtSlot(list,str,int,int)
    def _new_vmag(self,data,status,system,sensor):
        ''' updates the vector mag plots '''
        print(data,status,system,sensor)
    
    @pyqtSlot(list,str,int,int)
    def _new_sonar(self,data,status,system,sensor):
        ''' updates the sonar display '''
        print(data,status,system,sensor)
        
    @pyqtSlot(list,str,int,int)
    def _new_lidar(self,data,status,system,sensor):
        ''' updates the lidar display '''
        print('$LIDAR',data,status,system,sensor)
        
    @pyqtSlot(list,str,int,int)
    def _new_com_mag(self,data,status,system,sensor):#tf,status):
        ''' updates the real time compensated value display '''
        print(data,status,system,sensor)
        
    @pyqtSlot(list,str,int,int)
    def _new_det_mag(self,data,status,system,sensor):#flag,strength,status):
        ''' updates the real time detection display '''
        print(data,status,system,sensor)
        
    @pyqtSlot(list,str,int,int)
    def _new_inv_mag(self,data,status,system,sensor):
        ''' updates the real time inversion display '''
        print(data,status,system,sensor)
        
    @pyqtSlot(list,str,int,int)
    def _new_trk_mag(self,data,status,system,sensor):
        ''' updates the track message '''
        print('RTTRK',data,status,system,sensor)
    
    @pyqtSlot(list,str,int,int)
    def _new_mcu(self,data,status,system,sensor):
        ''' updates the mcu time '''
        print(data,status,system,sensor)
    
    @pyqtSlot()
    def _new_timeout(self):
        ''' lets the user know if there is a socket timeout '''
        print('Timeout')
        
if __name__ == '__main__':
    app = QApplication(sys.argv)
    aw = test()
    aw.show()
    app.exec_()
    


'''
Created on Aug 17, 2018

@author: reynolds
'''

from serial import Serial
from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import QGridLayout, QFrame, QWidget,QApplication, QPushButton, QLabel, QComboBox,\
    QSpacerItem, QSizePolicy
from PyQt5.QtCore import Qt
from PyQt5.Qt import QLineEdit, QTimer
from wrtdk.data.state.state import state
from wrtdk.qt5.animations import CubeWidget
from wrtdk.qt5.plots import PlotParameters, Plot2DWidget, Plot3DWidget

import sys, random, os
import numpy as np
from wrtdk.qt5.widgets import SoftwareLED, wrt_widget, FHLabelledInputWidget

class HealthWidget(wrt_widget):
    ''' a widget for displaying the health status of the sensor '''
    
    def __init__(self,parent=None,row=None,col=None,timeout=5000):
        ''' constructor '''
        super().__init__(parent)
        
        if row is None: row = range(1,10)
        if col is None: col = [0 for _ in range(10)]
        
        self.timeout = timeout
        
        # initialize widgets
        self.sys = self._getQLineEdit()
        self.syst = QTimer()
        self.syss = state()
        self.syst.timeout.connect(lambda: self.setSystemState('NC'))
        self.gpgga = self._getQLineEdit()
        self.gpggat = QTimer()
        self.gpggas = state()
        self.gpggat.timeout.connect(lambda: self.setGPGGAState('NC'))
        self.gpvtg = self._getQLineEdit()
        self.gpvtgt = QTimer()
        self.gpvtgs = state()
        self.gpvtgt.timeout.connect(lambda: self.setGPVTGState('NC'))
        self.gprmc = self._getQLineEdit()
        self.gprmct = QTimer()
        self.gprmcs = state()
        self.gprmct.timeout.connect(lambda: self.setGPRMCState('NC'))
        self.gphdt = self._getQLineEdit()
        self.gphdtt = QTimer()
        self.gphdts = state()  
        self.gphdtt.timeout.connect(lambda: self.setGPHDTState('NC'))
        self.gpvhw = self._getQLineEdit()
        self.gpvhwt = QTimer()
        self.gpvhws = state()
        self.gpvhwt.timeout.connect(lambda: self.setGPVHWState('NC'))
        self.vmag = self._getQLineEdit()
        self.vmagt = QTimer()
        self.vmags = state()
        self.vmagt.timeout.connect(lambda: self.setVMAGState('NC'))
        self.imu = self._getQLineEdit()
        self.imut = QTimer()
        self.imus = state()
        self.imut.timeout.connect(lambda: self.setIMUState('NC',0,0,0,0))
        self.sonar = self._getQLineEdit()
        self.sonart = QTimer()
        self.sonars = state()
        self.sonart.timeout.connect(lambda: self.setSonarState("NC"))
        self.lidar = self._getQLineEdit()
        self.lidart = QTimer()
        self.lidars = state()
        self.lidart.timeout.connect(lambda: self.setLidarState("NC"))
        self.pltfm = self._getQLineEdit()
        self.pltfms = state()
        self.gprip = self._getQLineEdit()
        self.gpript = QTimer()
        self.gprips = state()
        self.gpript.timeout.connect(lambda: self.setGPRIPState("NC"))
        
        # layout widget
        l = QGridLayout()
        #Always by itself top left.
        l.addWidget(self._getLabel('System Health',10,Qt.AlignCenter),0,0,1,4)
        #Always by itself top left.
        l.addWidget(self._getLabel('SYS:'),0,4,1,1)
        l.addWidget(self.sys,0,5,1,1)
        if(row[0]!=-1):
            l.addWidget(self._getLabel('PLTFM:'),row[0],2*col[0],1,1)
            l.addWidget(self.pltfm,row[0],2*col[0]+1,1,1)
        #row[0]
        #adding -1 for DO NOT INCLUDES
        if(row[1]!=-1):
            l.addWidget(self._getLabel('VMAG:'),row[1],2*col[1],1,1)
            l.addWidget(self.vmag,row[1],2*col[1]+1,1,1)
        if(row[2]!=-1):
            l.addWidget(self._getLabel('GPGGA:'),row[2],2*col[2],1,1)
            l.addWidget(self.gpgga,row[2],2*col[2]+1,1,1)
        if(row[3]!=-1):
            l.addWidget(self._getLabel('GPRMC:'),row[3],2*col[3],1,1)
            l.addWidget(self.gprmc,row[3],2*col[3]+1,1,1)
        if(row[4]!=-1):
            l.addWidget(self._getLabel('GPVTG:'),row[4],2*col[4],1,1)
            l.addWidget(self.gpvtg,row[4],2*col[4]+1,1,1)
        if(row[5]!=-1):
            l.addWidget(self._getLabel('GPVHW:'),row[5],2*col[5],1,1)
            l.addWidget(self.gpvhw,row[5],2*col[5]+1,1,1)
        if(row[6]!=-1):
            l.addWidget(self._getLabel('GPHDT:'),row[6],2*col[6],1,1)
            l.addWidget(self.gphdt,row[6],2*col[6]+1,1,1)
        if(row[7]!=-1):
            l.addWidget(self._getLabel('IMU:'),row[7],2*col[7],1,1)
            l.addWidget(self.imu,row[7],2*col[7]+1,1,1)
        if(row[8]!=-1):
            l.addWidget(self._getLabel('SONAR:'),row[8],2*col[8],1,1)
            l.addWidget(self.sonar,row[8],2*col[8]+1,1,1)
        if(row[9]!=-1):
            l.addWidget(self._getLabel('LIDAR:'),row[9],2*col[9],1,1)
            l.addWidget(self.lidar,row[9],2*col[9]+1,1,1)
        if(row[11]!=-1):
            l.addWidget(self._getLabel('GPRIP:'),row[11],2*col[11],1,1)
            l.addWidget(self.gprip,row[11],2*col[11]+1,1,1)
        l.addItem(QSpacerItem(10,10,QSizePolicy.Expanding,QSizePolicy.Expanding),max(row)+1,0,1,col[-1])
        #l.setVerticalSpacing(10)
        self.setLayout(l)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
    def _getLabel(self,text='',size=8,align=Qt.AlignRight|Qt.AlignVCenter):
        ''' sets the label font and size '''
        lbl = QLabel(text)
        lbl.setFont(QtGui.QFont('Times',size,QtGui.QFont.Bold))
        lbl.setAlignment(align)
        return lbl
        
    def _getQLineEdit(self,text='',readonly=True):
        ''' sets the line edit '''
        txt = QLineEdit()
        txt.setReadOnly(readonly)
        return txt
    
    def setStarted(self):
        ''' starts the widge '''
        self.syst.start(self.timeout)
        self.gpggat.start(self.timeout)
        self.gprmct.start(self.timeout)
        self.gpvtgt.start(self.timeout)
        self.gphdtt.start(self.timeout)
        self.gpvhwt.start(self.timeout)
        self.vmagt.start(self.timeout)
        self.imut.start(self.timeout)
        self.lidart.start(self.timeout)
        self.sonart.start(self.timeout)
        self.gpript.start(self.timeout)
    
    def setPlatformStarted(self):
        ''' sets the started platform state '''
        self.setPlatformState('AG')
        
    def setPlatformStopped(self):
        ''' sets the stoped platform state '''
        self._reset(self.pltfm)
        self.pltfms.reset()
        
    def setStopped(self):
        ''' resets the entire widget '''
        self.syst.stop()
        self._reset(self.sys)
        self.syss.reset()
        self.gpggat.stop() 
        self._reset(self.gpgga)
        self.gpggas.reset()
        self.gprmct.stop()
        self._reset(self.gprmc)
        self.gprmcs.reset()
        self.gpvtgt.stop()
        self._reset(self.gpvtg)
        self.gpvtgs.reset()
        self.gphdtt.stop()
        self._reset(self.gphdt)
        self.gphdts.reset()
        self.gpvhwt.stop()
        self._reset(self.gpvhw)
        self.gpvhws.reset()
        self.vmagt.stop()
        self._reset(self.vmag)
        self.vmags.reset()
        self.imut.stop()
        self._reset(self.imu)
        self.imus.reset()
        self.lidart.stop()
        self._reset(self.lidar)
        self.lidars.reset()
        self.sonart.stop()
        self._reset(self.sonar)
        self.sonars.reset()
        self._reset(self.gprip)
        self.gprips.reset()
        
    def _reset(self,widget):
        ''' resets the widget '''
        widget.setText('')
        widget.setStyleSheet('')
    
    def _resett(self,timer):
        ''' resets the timer '''
        #timer.stop()
        timer.start(self.timeout)
    
    def setVMAGState(self,state=''):
        ''' sets the state of the vector mag '''
        self._resett(self.vmagt)
        if not self.vmags.isDifferent(state): return
        self._setState(self.vmag,state)
        
    def setGPGGAState(self,state):
        ''' sets the state of the gpgga '''
        self._resett(self.gpggat)
        if not self.gpggas.isDifferent(state): return
        self._setState(self.gpgga, state)
        
    def setGPRMCState(self,state):
        ''' sets the state of the gprmc '''
        self._resett(self.gprmct)
        if not self.gprmcs.isDifferent(state): return
        self._setState(self.gprmc, state)
        
    def setGPVTGState(self,state):
        ''' sets the state of the gpvtg '''
        self._resett(self.gpvtgt)
        if not self.gpvtgs.isDifferent(state): return
        self._setState(self.gpvtg, state)
        
    def setGPHDTState(self,state):
        ''' sets the state of the gphdt '''
        self._resett(self.gphdtt)
        if not self.gphdts.isDifferent(state): return
        self._setState(self.gphdt, state)
        
    def setGPVHWState(self,state):
        ''' sets the state of the gpvhw '''
        self._resett(self.gpvhwt)
        if not self.gpvhws.isDifferent(state): return
        self._setState(self.gpvhw, state)
        
    def setSystemState(self,state):
        ''' sets the state of the system '''
        self._resett(self.syst)
        if not self.syss.isDifferent(state): return
        self._setState(self.sys, state)
        
    def setSonarState(self,state):
        ''' sets the state of the sonar sensor '''
        self._resett(self.sonart)
        if not self.sonars.isDifferent(state): return
        self._setState(self.sonar, state)
        
    def setLidarState(self,state):
        ''' sets the state of the lidar sensor '''
        self._resett(self.lidart)
        if not self.lidars.isDifferent(state): return
        self._setState(self.lidar, state)
        
    def setIMUState(self,state,imu=0,acc=0,gyr=0,mag=0):
        ''' sets the state of the Microstrain IMU '''
        self._resett(self.imut)
        if not self.imus.isDifferent(state): return
        self._setState(self.imu, state)
        #self.imu.setText('%s %d%d%d%d'%(self.imu.text(),imu,acc,gyr,mag))# uncomment for BNO055 IMU
        
    def setPlatformState(self,state):
        ''' sets the platform state '''
        # there is no timer for  this software led
        if not self.pltfms.isDifferent(state): return
        self._setState(self.pltfm, state)
        
    def setGPRIPState(self,state):
        ''' sets the state of the gprip message '''
        self._resett(self.gpript)
        if not self.gprips.isDifferent(state): return
        self._setState(self.gprip, state)
class altQualWidget(QFrame):
    def __init__(self, parent=None):
        QFrame.__init__(self,parent=None)
        self.n=1
        self.sensors={}
        
        self.l=QGridLayout()
        lab=QLabel('ID:')
        alt=QLabel('Altitude:')
        quality=QLabel('Quality:')
        self.l.addWidget(lab,0,0,1,1)
        self.l.addWidget(alt,1,0,1,1)
        self.l.addWidget(quality,2,0,1,1)
        self.setLayout(self.l)
    def addSensor(self,ID,alt,quality='NA'):
        self.sensors[ID]=QLabel(str(round(alt,2))),QLabel(str(quality))
        self.l.addWidget(QLabel(ID),         0,self.n,1,1)
        self.l.addWidget(self.sensors[ID][0],1,self.n,1,1)
        self.l.addWidget(self.sensors[ID][1],2,self.n,1,1)
        self.n+=1
    def setAlt(self,ID,alt):
        self.sensors[ID][0].setText(str(round(alt,2)))
    def setQuality(self,ID,quality):
        self.sensors[ID][1].setText(str(quality))
                                   
class MapWidget(QFrame):
    ''' for multiple sensors '''
    
    def __init__(self,parent=None,lsize=10,tsize=16):
        ''' constructor '''
        QFrame.__init__(self,parent)
        self.alts=altQualWidget()
        self.syms=['o','^','s','*']
        self.sensors=[]
        self.num=0
        self.plot = Plot2DWidget(parameters = 
                                 PlotParameters('UTM Map',
                                    'Easting (m)',
                                    'Northing (m)',
                                    lsize=lsize,
                                    tsize=tsize),
                                    lines=0)
        self.plot.set_aspect()
        self.leg=[]
        self.track=1;
        self.bxsize=10;
#         self.plot.add_line([], [],'ko-')#black line
#         self.plot.add_line([],[],'go')  #green dot
#         self.plot.set_legend(['Path','Current','Track'])
        self.targets=0
        l = QGridLayout()
        l.addWidget(self.alts,0,0)
        l.addWidget(self.plot,1,0)
        
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
    def setupTargets(self,x,y,IDs):
        self.targets=1
        self.plot.add_line([],[],'ro')
        self.leg.append('Targets')
        for i in range(len(IDs)):
            self.plot.append(x,y,line=0,autoscale=False)
            
    def addsensor(self,name):
        self.sensors.append(str(name))
        self.plot.add_line([],[],'k'+self.syms[self.num]+'-')
        self.plot.add_line([],[],'g'+self.syms[self.num])
        self.leg.append('Sensor '+str(self.sensors[self.num])+' path')
        self.leg.append('Sensor '+str(self.sensors[self.num])+' current')
        self.plot.set_legend(self.leg)
        self.num+=1
    def appendPath(self,name,x,y,z):
        for i in range(len(self.sensors)):
            if self.sensors[i]==name:
                self.plot.append(x,y,line=2*i+self.targets,autoscale=True)
                self.plot.set_data(x,y,2*i+self.targets+1,autoscale=True)
                
    def setTrack(self,sensor,bxsize=10):
        for i in range(len(self.sensors)):
            if self.sensors[i]==sensor:
                self.track=i
        self.bxsize=bxsize
                
    def clear(self):
        ''' clears all the plots, text boxes and software LEDs '''
        print("CLEARING MAP")
        for i in range(self.plot.line_total()-1,3,-1):
            self.plot.remove_line(i)
        for i in range(self.num):
            self.plot.clear_line(i)
        self.num=0
        self.plot.clear()

class SystemWidget(QFrame):
    ''' 
    a class for connecting and logging data form a system
    '''
    
    def __init__(self,parent=None):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        # define widgets
        font = QtGui.QFont('Times',12,QtGui.QFont.Bold)
        self.connect = QPushButton('Connect')
        self.connect.setStyleSheet('background-color: green')
        self.connect.setFont(font)
        self.output = LogWidget(self)
        self.output.setEnabled(False)
        self.timestamp = FHLabelledInputWidget(self,'MCU Time:','',0)
        self.timestamp.setFont(QtGui.QFont('Times',8,QtGui.QFont.Bold))
        self.timestamp.widget.setReadOnly(True)
        
        # layout grid
        l = QGridLayout()
        l.addWidget(self.connect,0,0,1,1)
        l.addWidget(self.timestamp,0,1,1,2)
        l.addWidget(self.output,1,0,1,2)
        l.setSpacing(0)
        l.setColumnStretch(0,1)
        l.setColumnStretch(1,3)
        self.setContentsMargins(0, 0, 0, 0)
        self.setLayout(l)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
        # define stat variables
        self._connected = False
        
    def isConnected(self):
        ''' returns whether the system is connected '''
        return self._connected
        
    def setConnected(self):
        ''' sets the widget in a connected state '''
        self.connect.setStyleSheet("background-color: red")
        self.connect.setText("Disconnect")
        self.output.setEnabled(True)
        self._connected = True
            
    def setDisconnected(self):
        ''' sets the widget in a disconnected state '''
        self.connect.setStyleSheet("background-color: green")
        self.connect.setText("Connect")
        self.output.setEnabled(False)
        self._connected = False

class LogWidget(QWidget):
    ''' a widget for logging collected data '''
    
    def __init__(self,parent=None,default='',lfnt=None,tfnt=None,align=None,
                 start_color='green',stop_color='red'):
        ''' constructor '''
        QWidget.__init__(self,parent)
        
        # define var
        self._line = 1
        self._startc = start_color
        self._stopc = stop_color
        
        # define widgets
        self.txt = self._get_qlineedit('',tfnt,align)
        self.line = self._get_qlineedit('%d'%self._line,lfnt, align)
        self.line.setReadOnly(True)
        self.log = self._get_qpushbutton('LOG',lfnt,align)
        self.pause = self._get_qpushbutton('PAUSE',lfnt,align)
        self.pause.setEnabled(False)
        self.pause.setStyleSheet('')
        plus = self._get_qpushbutton('+',lfnt,align)
        plus.clicked.connect(self._plus)
        minus = self._get_qpushbutton('-',lfnt,align)
        minus.clicked.connect(self._minus)
        self.log.setStyleSheet('background-color: %s'%self._startc)
        
        # layout widgets
        l = QGridLayout()
        l.addWidget(self.txt,0,0,1,3)
        l.addWidget(minus,1,0,1,1)
        l.addWidget(plus,1,1,1,1)
        l.addWidget(self.line,1,2,1,1)
        l.addWidget(self.log,0,3,1,1)
        l.addWidget(self.pause,1,3,1,1)
        l.setColumnStretch(0,1)
        l.setColumnStretch(1,1)
        l.setColumnStretch(2,1)
        l.setColumnStretch(3,1)
        self.setLayout(l)
        
        # define state variables
        self._log = False
        self._pause = False
        
    def _get_qlineedit(self,txt='',fnt=None,align=None):
        ''' creates the qlabels '''
        l = QLineEdit(txt)
        if fnt is not None: l.setFont(fnt)
        if align is not None: l.setAlignment(align)
        return l
        
    def _get_qpushbutton(self,txt='',fnt=None,align=None):
        ''' creates a qpush button '''
        b = QPushButton(txt)
        if fnt is not None: b.setFont(fnt)
        if align is not None: b.setAlignment(align)
        return b
        
    def setEnabled(self,enabled=False):
        ''' sets the enabled state of the logger '''
        self.log.setEnabled(enabled)
        self.txt.setReadOnly(not enabled)
        
    def isLogging(self):
        ''' returns whether the widget is in log mode or not '''
        return self._log

    def isPaused(self):
        return self._pause
    
    def setStarted(self):
        ''' sets the widget in logging mode '''
        self.log.setStyleSheet('background-color: %s'%self._stopc)
        self.log.setText('STOP')
        self._log = True
        #self.pause.setStyleSheet('background-color: %s'%self._stopc)

    def setStopped(self):
        ''' sets the widget in not logging mode '''
        self.log.setStyleSheet('background-color: %s'%self._startc)
        self.log.setText('LOG')
        self._log = False
        self._plus()
        self.pause.setEnabled(False)
        self.pause.setStyleSheet('')
        if self._pause: self.setResumed()
        
    def setPaused(self):
        self.log.setStyleSheet('background-color: red')
        self.log.setText('RESUME')
        
    def setResumed(self):
        self.log.setStyleSheet('background-color: green')
        self.log.setText('PAUSE')

    def _plus(self):
        self._line += 1
        self.line.setText('%d' % self._line)
    
    def _minus(self):
        if self._line > 1: self._line -= 1
        self.line.setText('%d' % self._line)
    
    def getFilename(self):
        ''' returns the current filename '''
        return '%s_%d' % (self.txt.text(),self._line)

class GPGGAWidget(QFrame):
    ''' a widget for displaying the GPGGA information '''
    def __init__(self,parent=None,rtk=False,lfnt=None,
                 tfnt=None,align=None,frame=QFrame.StyledPanel | QFrame.Plain):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        # create the widgets
        self._time = self._get_qlineedit(tfnt,align)
        self._lat = self._get_qlineedit(tfnt,align)
        self._lon = self._get_qlineedit(tfnt,align)
        self._fix = self._get_qlineedit(tfnt,align)
        self._n = self._get_qlineedit(tfnt,align)
        self._dop = self._get_qlineedit(tfnt,align)
        
        #setup the ui
        l = QGridLayout()
        l.addWidget(self._get_qlabel('UTC Time',lfnt,align),0,0,1,1)
        l.addWidget(self._get_qlabel('Latitude',lfnt,align),0,1,1,1)
        l.addWidget(self._get_qlabel('Longitude',lfnt,align),0,2,1,1)
        l.addWidget(self._get_qlabel('N Sats',lfnt,align),0,3,1,1)
        l.addWidget(self._get_qlabel('Quality',lfnt,align),0,4,1,1)
        l.addWidget(self._get_qlabel('DOP',lfnt,align),0,5,1,1)
        l.addWidget(self._time,1,0,1,1)
        l.addWidget(self._lat,1,1,1,1)
        l.addWidget(self._lon,1,2,1,1)
        l.addWidget(self._fix,1,3,1,1)
        l.addWidget(self._n,1,4,1,1)
        l.addWidget(self._dop,1,5,1,1)
        l.setSpacing(2)
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(frame)
        
        # initialize state variables
        self._dop_state = state()
        self._fix_state = state()
        self._rtk = rtk
        
    def set_data(self,time='',lat=360,lon=360,n=0,fix=0,dop=np.NaN):
        ''' sets the data on the plot and appends to the map '''
        self._time.setText(time)
        self._lat.setText('%.6f'%lat)
        self._lon.setText('%.6f'%lon)
        self._n.setText('%d'%n)
        self._set_fix(fix)
        self._set_dop(dop)
        
    def clear(self):
        ''' reset all the text boxes '''
        self._time.setText('')
        self._lat.setText('')
        self._lon.setText('')
        self._n.setText('')
        self._fix.setText('')
        self._fix.setStyleSheet('')
        self._dop.setText('')
        self._dop.setStyleSheet('')
        
    def _set_fix(self,fix=1):
        ''' sets the fix of the gnss receiver '''
        
        if not self._fix_state.isDifferent(fix): return
        
        self._fix.setText('%d'%fix)# update the text
        if self._rtk:
            if fix == 4: self._fix.setStyleSheet('background-color: green')
            else: self._fix.setStyleSheet('background-color: red')
            
    def _set_dop(self,dop=100):
        ''' sets the dop text and warning '''
        if not self._dop_state.isDifferent(dop): return
        
        self._dop.setText('%.2f' % dop)# update the text
        if not self._rtk:
            if dop >= 2: self._dop.setStyleSheet('background-color: red')
            else: self._dop.setStyleSheet('background-color: green')
        
    def _get_qlabel(self,text,fnt,algn):
        ''' creates the labels '''
        l = QLabel(text)
        if fnt is not None: l.setFont(fnt)
        if algn is not None: l.setAlignment(algn)
        return l
        
    def _get_qlineedit(self,fnt,algn):
        ''' creates the line edits '''
        le = QLineEdit()
        le.setReadOnly(True)
        if fnt is not None: le.setFont(fnt)
        if algn is not None: le.setAlignment(algn)
        return le

class IMUVisualWidget(QFrame):
    
    def __init__(self,parent=None,px=.2,py=.2,pz=.2,tfnt=None):
        QFrame.__init__(self,parent)
        self.roll = self._get_txt('Roll:',tfnt)
        self.pitch = self._get_txt('Pitch:',tfnt)
        self.yaw = self._get_txt('Yaw:',tfnt)
        self.imu = CubeWidget(self,px=px,py=py,pz=pz)
        
        l = QGridLayout()
        l.addWidget(self.imu,0,0,1,3)
        l.addWidget(self.roll,1,0,1,1)
        l.addWidget(self.pitch,1,1,1,1)
        l.addWidget(self.yaw,1,2,1,1)
        
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
    def _get_txt(self,text='',fnt=None):
        txt = QLineEdit(text)
        txt.setReadOnly(True)
        if fnt is not None: txt.setFont(fnt)
        return txt
    
    def setEuler(self,roll,pitch,yaw):
        self.roll.setText('Roll: %.2f' % roll)
        self.pitch.setText('Pitch: %.2f' % pitch)
        self.yaw.setText('Yaw: %.2f' % yaw)
        self.imu.update(roll,pitch,yaw)
        
    def clear(self):
        ''' clears the IMU widget '''
        self.setEuler(0,0,0)
        
class GCSMapWidget(QFrame):
    ''' a class for a GCS map '''
    
    def __init__(self,parent=None,lines=1,leg=['Path'],lsize=10,tsize=16):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        # setup the plot
        self.plot = Plot2DWidget(self,
                                 parameters=PlotParameters('GCS Map',
                                                           'Longitude (deg)',
                                                           'Latitude (deg)',
                                                           lsize=lsize,
                                                           tsize=tsize),
                                 lines=lines)
        self.plot.axes.set_aspect('equal')
        self.plot.set_linestyle(line=0,style='None')
        self.plot.set_marker(line=0,marker='s')
        self.plot.set_legend(leg)
        
        #layout the map
        l = QGridLayout()
        l.setSpacing(0)
        l.addWidget(self.plot)
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        self.setStyleSheet('background-color: white')
        
    def append(self,x=0,y=0,line=0):
        ''' appends to the plot data '''
        self.plot.append(x,
                         y,
                         line,
                         autoscale=True)

class UTMMapWidget(QFrame):
    ''' a class for a UTM map '''
    
    def __init__(self,parent=None,lines=1,leg=['Path'],lsize=10,tsize=16,xref=0,yref=0):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        # setup the reference coordinates
        self._x_ref = xref
        self._y_ref = yref
        
        # setup the plot
        parameters = PlotParameters('UTM Map',
                                    'Easting - %.2f (m)' % self._x_ref,
                                    'Northing - %.2f (m)' % self._y_ref,
                                    lsize=lsize,
                                    tsize=tsize)
        self.plot = Plot2DWidget(parent=self,parameters=parameters,lines=lines)
        self.plot.axes.set_aspect('equal')
        self.plot.set_linestyle(line=0,style='None')
        self.plot.set_marker(line=0,marker='s')
        self.plot.set_legend(leg)
        
        #layout the map
        l = QGridLayout()
        l.setSpacing(0)
        l.addWidget(self.plot)
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        self.setStyleSheet('background-color: white')
        
    def setReference(self,x=0,y=0):
        ''' sets the reference coordinates '''
        self._x_ref = x
        self._y_ref = y
        self.plot.axes.set_xlabel('Easting - %.2f (m)' % self._x_ref)
        self.plot.axes.set_xlabel('Northing - %.2f (m)' % self._y_ref)
        
    def append(self,x=0,y=0,line=0):
        ''' appends to the plot '''
        self.plot.append(x-self._x_ref,
                         y-self._x_ref,
                         line,
                         autoscale=True)

class GPGGAVisualWidget(QFrame):
    ''' a class for visually displaying the NMEA string '''
    
    def __init__(self,parent=None,rtk=False,lfnt=None,tfnt=None,align=None,
                 lsize=10,tsize=16):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        # create the widgets
        self._gga = GPGGAWidget(self,rtk=rtk,
                                tfnt=tfnt,lfnt=lfnt,
                                align=align,frame=QFrame.Plain)
        self._map = GCSMapWidget(self,lsize=lsize,tsize=tsize)# map widget
        
        #setup the ui
        l = QGridLayout()
        l.addWidget(self._gga)
        l.addWidget(self._map,2,0,6,6)
        l.setSpacing(2)
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
        # initialize state variables
        self._dop_state = state()
        self._fix_state = state()
        self._rtk = rtk
        
    def set_data(self,time='',lat=360,lon=360,n=0,fix=0,dop=np.NaN,append=True):
        ''' sets the data on the plot and appends to the map '''
        self._gga.set_data(time, lat, lon, n, fix, dop)
        if append: self._map.append(lon,lat)
    
class RTMag3DWidget(QFrame):
    ''' a 3d display of the real time inversion '''
    
    def __init__(self,parent=None,lsize=10,tsize=16):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        self.det = SoftwareLED(self,'DETECTION')
        _curr_m = self._get_qlabel(txt='Moment',align=Qt.AlignCenter)
        self.curr_m = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _track_m = self._get_qlabel(txt='Track Moment',align=Qt.AlignCenter)
        self.track_m = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _source_n = self._get_qlabel(txt='Sources',align=Qt.AlignCenter)
        self.source_n = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _state = self._get_qlabel(txt='State',align=Qt.AlignCenter)
        self.state = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _alt = self._get_qlabel('Altitude (m)',align=Qt.AlignCenter)
        self.altitude = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        
        self.plot = Plot3DWidget(parameters=
                                 PlotParameters(xlabel='x(m)',
                                                ylabel='y(m)',
                                                zlabel='z(m)',
                                                lsize=lsize,
                                                tsize=tsize),
                                 lines=0)
        self.plot.add_line([], [], [],'ko-')
        self.plot.add_line([],[],[],'go')
        self.plot.add_line([],[],[],'bx',markersize=14,markeredgewidth=8)
        self.plot.add_line([], [], [],'mo-')
        self.plot.set_legend(['Path','Current','Track','Source'])
        
        l = QGridLayout()
        l.addWidget(self.det,0,0,2,1)
        l.addWidget(_curr_m,0,1,1,1)
        l.addWidget(self.curr_m,1,1,1,1)
        l.addWidget(_source_n,0,2,1,1)
        l.addWidget(self.source_n,1,2,1,1)
        l.addWidget(_track_m,0,3,1,1)
        l.addWidget(self.track_m,1,3,1,1)
        l.addWidget(_state,0,4,1,1)
        l.addWidget(self.state,1,4,1,1)
        l.addWidget(_alt,0,5,1,1)
        l.addWidget(self.altitude,1,5,1,1)
        l.addWidget(self.plot,2,0,1,9)
        l.setRowStretch(0,1)
        l.setRowStretch(1,1)
        l.setRowStretch(2,10)
        
        l.setSpacing(2)
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
    def _get_qlabel(self,txt='',font=None,align=None):
        ''' creates the qlabels '''
        l = QLabel(txt)
        if font is not None: l.setFont(font)
        if align is not None: l.setAlignment(align)
        return l
        
    def _get_qlineedit(self,txt='',font=None,align=None,readonly=False):
        ''' creates the qlabels '''
        l = QLineEdit(txt)
        if font is not None: l.setFont(font)
        if align is not None: l.setAlignment(align)
        l.setReadOnly(readonly)
        return l
    
    def clear(self):
        ''' clears all the plots, text boxes and software LEDs '''
        for i in range(self.plot.line_total()-1,3,-1): self.plot.remove_line(i)
        self.plot.clear_line(0)
        self.plot.clear_line(1)
        self.plot.clear_line(2)
        self.curr_m.setText('')
        self.source_n.setText('')
        self.track_m.setText('')
        self.altitude.setText('')
        self.det.reset()
    
    def appendPath(self,x,y,z):
        ''' appends to the path line '''
        self.plot.append(x,y,z,line=0,autoscale=True)
        self.plot.set_data(x,y,z,line=1,autoscale=True)
        self.altitude.setText('%.2f' % z)
        
    def appendTrack(self,x,y,z):
        ''' appends to the mean inversion locations '''
        self.plot.append(x,y,z,line=2,autoscale=True)
        
    def addSource(self,x,y,z):
        ''' adds an inversion line '''
        self.plot.add_line(x,y,z,'mo-')
        
    def setTrackMoment(self,m=-1):
        ''' sets the track moment '''
        self.track_m.setText('%.2f' % m)
        
    def setCurrentMoment(self,m=-1):
        ''' sets the current moment '''
        self.curr_m.setText('%.2f' % m)
        
    def setTotalSources(self,n=-1):
        self.source_n.setText('%d' % n)
        
    def setDetection(self,detection=False):
        ''' sets the detection state '''
        if detection: self.det.setBad()
        else: self.det.reset()
        
    def setState(self,string=''):
        ''' relays the state of the sim board '''
        self.state.setText(string)
        
class RTMag2DWidget(QFrame):
    ''' a 2d representation of the inversion '''
    
    def __init__(self,parent=None,lsize=10,tsize=16):
        ''' constructor '''
        QFrame.__init__(self,parent)
        
        self.det = SoftwareLED(self,'DETECTION')
        _curr_m = self._get_qlabel(txt='Current Moment',align=Qt.AlignCenter)
        self.curr_m = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _track_m = self._get_qlabel(txt='Track Moment',align=Qt.AlignCenter)
        self.track_m = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _source_n = self._get_qlabel(txt='Sources',align=Qt.AlignCenter)
        self.source_n = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _state = self._get_qlabel(txt='State',align=Qt.AlignCenter)
        self.state = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        _alt = self._get_qlabel('Altitude (m)',align=Qt.AlignCenter)
        self.altitude = self._get_qlineedit(readonly=True,align=Qt.AlignCenter)
        
        self.plot = Plot2DWidget(parameters = 
                                 PlotParameters('UTM Map',
                                    'Easting (m)',
                                    'Northing (m)',
                                    lsize=lsize,
                                    tsize=tsize),
                                    lines=0)
        self.plot.set_aspect()
        self.plot.add_line([], [],'ko-')
        self.plot.add_line([],[],'go')
        self.plot.add_line([],[],'bx',markersize=14,markeredgewidth=8)
        self.plot.add_line([], [],'mo-')
        self.plot.set_legend(['Path','Current','Track','Source'])
        
        l = QGridLayout()
        l.addWidget(self.det,0,0,2,1)
        l.addWidget(_curr_m,0,1,1,1)
        l.addWidget(self.curr_m,1,1,1,1)
        l.addWidget(_source_n,0,2,1,1)
        l.addWidget(self.source_n,1,2,1,1)
        l.addWidget(_track_m,0,3,1,1)
        l.addWidget(self.track_m,1,3,1,1)
        l.addWidget(_state,0,4,1,1)
        l.addWidget(self.state,1,4,1,1)
        l.addWidget(_alt,0,5,1,1)
        l.addWidget(self.altitude,1,5,1,1)
        l.addWidget(self.plot,2,0,1,9)
        l.setRowStretch(0,1)
        l.setRowStretch(1,1)
        l.setRowStretch(2,10)
        
        l.setSpacing(2)
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
    def _get_qlabel(self,txt='',font=None,align=None):
        ''' creates the qlabels '''
        l = QLabel(txt)
        if font is not None: l.setFont(font)
        if align is not None: l.setAlignment(align)
        return l
        
    def _get_qlineedit(self,txt='',font=None,align=None,readonly=False):
        ''' creates the qlabels '''
        l = QLineEdit(txt)
        if font is not None: l.setFont(font)
        if align is not None: l.setAlignment(align)
        l.setReadOnly(readonly)
        return l
    
    def clear(self):
        ''' clears all the plots, text boxes and software LEDs '''
        for i in range(self.plot.line_total()-1,3,-1):self.plot.remove_line(i)
        self.plot.clear_line(0)
        self.plot.clear_line(1)
        self.plot.clear_line(2)
        self.curr_m.setText('')
        self.source_n.setText('')
        self.track_m.setText('')
        self.altitude.setText('')
        self.det.reset()
    
    def appendPath(self,x,y,z):
        ''' appends to the path line '''
        self.plot.append(x,y,line=0,autoscale=True)
        self.plot.set_data(x,y, line=1, autoscale=True)
        self.altitude.setText('%.2f' % z)
        
    def appendTrack(self,x,y,z):
        ''' appends to the mean inversion locations '''
        self.plot.append(x,y,line=2,autoscale=True)
        
    def addSource(self,x,y,z):
        ''' adds an inversion line '''
        self.plot.add_line(x,y,'mo-')
        
    def setTrackMoment(self,m=-1):
        ''' sets the track moment '''
        self.track_m.setText('%.2f' % m)
        
    def setCurrentMoment(self,m=-1):
        ''' sets the current moment '''
        self.curr_m.setText('%.2f' % m)
        
    def setTotalSources(self,n=-1):
        self.source_n.setText('%d' % n)
        
    def setDetection(self,detection=False):
        ''' sets the detection state '''
        if detection: self.det.setBad()
        else: self.det.reset()
        
    def setState(self,string=''):
        ''' displays the state of the sim board '''
        self.state.setText(string)

class dataport_widget(RTMag3DWidget):
    
    def __init__(self,parent=None,lfnt=None,tfnt=None,title=''):
        QFrame.__init__(self,parent)
        
        self.address = self._get_qlineedit(font=lfnt)
        self.port = self._get_qlineedit(font=lfnt)
        self.baud = QComboBox()
        self.baud.addItems(list(map(str,Serial.BAUDRATES)))
        self.baud.setCurrentText('9600')
        self.type = QComboBox()
        self.type.currentTextChanged.connect(self._new_commstyle)
        self.type.addItems(['UDP','TCP','Serial'])
        
        # layout the widget
        l = QGridLayout()
        l.addWidget(self._get_qlabel(title,font=tfnt,align=Qt.AlignCenter),0,0,1,2)
        l.addWidget(self._get_qlabel('Type:',font=lfnt,align=Qt.AlignRight),1,0,1,1)
        l.addWidget(self.type,1,1,1,1)
        l.addWidget(self._get_qlabel('Address:',font=lfnt,align=Qt.AlignRight),2,0,1,1)
        l.addWidget(self.address,2,1,1,1)
        l.addWidget(self._get_qlabel('Port:',font=lfnt,align=Qt.AlignRight),3,0,1,1)
        l.addWidget(self.port,3,1,1,1)
        l.addWidget(self._get_qlabel('Baudrate:',font=lfnt,align=Qt.AlignRight),4,0,1,1)
        l.addWidget(self.baud,4,1,1,1)
        l.addItem(QSpacerItem(20,40,QSizePolicy.Expanding,QSizePolicy.Expanding),5,0,1,2)
        
        self.setLayout(l)
        self.setContentsMargins(0, 0, 0, 0)
        self.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        
    def get_port(self):
        ''' returns the port type, address and port or baud '''
        txt = self.type.currentText()
        if txt == 'UDP' or txt == 'TCP':
            try:
                return (txt,self.address.text(),int(self.port.text()))
            except Exception as e:
                exc_type, _, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
                return None
        elif txt == 'Serial':
            return (txt,self.port.text(),int(self.baud.currentText()))
        else: return None
        
    def _serial_enabled(self,enabled=False):
        ''' sets the serial model enabled or not '''
        self.address.setReadOnly(enabled)
        self.port.setReadOnly(False)
        self.baud.setEnabled(enabled)
            
    def _new_commstyle(self,value):
        ''' handles a change in the type combobox '''
        if value == 'UDP' or value == 'TCP': self._serial_enabled(False)
        elif value == 'Serial': self._serial_enabled(True)
        
    def _get_qlabel(self,txt='',font=None,align=None):
        ''' creates the qlabels '''
        l = QLabel(txt)
        if font is not None: l.setFont(font)
        if align is not None: l.setAlignment(align)
        return l
        
    def _get_qlineedit(self,txt='',font=None,align=None,readonly=False):
        ''' creates the qlabels '''
        l = QLineEdit(txt)
        if font is not None: l.setFont(font)
        if align is not None: l.setAlignment(align)
        l.setReadOnly(readonly)
        return l

class MainWindow(QWidget):

    def __init__(self):
        super(MainWindow, self).__init__()
        self.w = LogWidget(self)
        clear = QPushButton('CLEAR')
        clear.clicked.connect(self._clear)
        self.nmea = GPGGAWidget(self)
        #self.imu = IMUVisualWidget(self)
        self.map = GPGGAVisualWidget(self,
                                     rtk=True,
                                     align=QtCore.Qt.AlignCenter|QtCore.Qt.AlignVCenter,
                                     lsize=10,
                                     tsize=16)
        self.utm = UTMMapWidget(self,lsize=10,tsize=16)
        self.gcs = GCSMapWidget(self,lsize=10,tsize=16)
        self.rtmag = RTMag3DWidget(self)
        self.port = dataport_widget(self,title='Communication Port')
        self.rtmag2d = RTMag2DWidget(self)
        layout = QGridLayout()
        layout.addWidget(self.w,0,0,1,2)
        layout.addWidget(clear,0,2,1,1)
        layout.addWidget(self.nmea,1,0,1,3)
        layout.addWidget(self.port,2,0,1,1)
        layout.addWidget(self.map,2,1,1,1)
        layout.addWidget(self.utm,2,2,1,1)
        layout.addWidget(self.gcs,3,0,1,1)
        layout.addWidget(self.rtmag2d,3,1,1,1)
        layout.addWidget(self.rtmag,3,2,1,1)
        for i in range(layout.columnCount()):
            layout.setColumnStretch(i,1)
        self.setLayout(layout)
        
        self.w.log.clicked.connect(self.onClick)
        
        self._det = False
        
    def _clear(self):
        ''' clears the real time plot '''
        self.rtmag.clear()
        self.rtmag2d.clear()
        
    def onClick(self):
        if self.w.isLogging(): 
            self.w.setStopped()
        else:
            print('filename: %s.dat'%self.w.getFilename())
            self.w.setStarted()
            
        #print('port:',self.port.get_port())
        
        self.map.set_data('1234',43.0,127.4,3,12,100)
        self.nmea.set_data('1234',43.0,127.4,3,8,100)
        
        #test utm
        self.utm.plot.set_data([1,2,3,4],[1,1,1,1],autoscale=True)
        self.utm.plot.append(5,3,autoscale=True)
        
        # test gcs
        self.gcs.append(0,2,line=0)
        self.gcs.append(1,2,line=0)
        
        self.rtmag2d.appendPath([random.random()],
                                    [random.random()],
                                    [random.random()])
        self.rtmag2d.appendTrack([random.random()],
                                    [random.random()],
                                    [random.random()])
        self.rtmag2d.addSource([random.random()],
                                    [random.random()],
                                    [random.random()])
        self.rtmag2d.setTrackMoment(random.random())
        self.rtmag2d.setCurrentMoment(random.random())
        self.rtmag2d.setTotalSources(random.randint(0,100))
        
        self.rtmag.appendPath([random.random()],
                                    [random.random()],
                                    [random.random()])
        
        self.rtmag.appendTrack([random.random()],
                                    [random.random()],
                                    [random.random()])
        
        self.rtmag.addSource([0,random.random()],
                                    [0,random.random()],
                                    [0,random.random()])
        
        self.rtmag.setTrackMoment(random.random())
        self.rtmag.setCurrentMoment(random.random())
        self.rtmag.setTotalSources(random.randint(0,100))
        
        self.rtmag.setDetection(self._det)
        self._det = not self._det

if __name__.startswith('__main__'):
    app = QApplication(sys.argv)
    window = MainWindow()
    window.setWindowTitle("Color Picker Demo")
    window.show()
    app.exec_()


'''
Created on Aug 17, 2018

@author: reynolds
'''

import sys, glob, serial, socket, time, select

class CommsPort(object):
    '''
    a super class for a communication port
    '''
    def __init__(self,port=''):
        ''' constructor '''
        self.set_name(port)
        self.port = None
        
    def set_name(self,name):
        self.name = name

    def open(self):
        ''' opens the port '''
        print('Opening ' + self.name + ' ... ',end='')
        pass

    def available(self):
        ''' checks to see if anything is available '''
        pass
    
    def read(self,n):
        pass
    
    def send(self,message):
        pass

    def close(self):
        ''' closes the port ''' 
        print('Closing ' + self.name + ' ... ',end='')
        try:
            self.port.close()
            print('Done')
            return True
        except Exception as e:
            print('Error. %s'%str(e))
            return False

    ''' reads a single line of ascii data '''
    def readLine(self):
        pass

    ''' checks to see if anything is open '''
    def isOpen(self):
        pass

class SerialPort(CommsPort):
    '''
    a serial data communication port
    '''
    def __init__(self,port=[],baud=9600,timeout=1):
        ''' Constructor '''
        super().__init__(port + '@' + str(baud))
        self.open(port,baud,timeout=timeout)

    def open(self,port,baud,timeout=0):
        self.set_name('%s@%d with %d s timeout'%(port,baud,timeout))
        ''' opens the serial port'''
        super().open()
        try:
            self.port = serial.Serial(port,baud,timeout=timeout)
            print('Done')
        except (OSError, serial.SerialException):
            print('Error. SerialException')

    def available(self):
        ''' checks to see if data is available '''
        super().available()
        return self.port.in_waiting
    
    def read(self,n=None):
        if n == None: n = self.port.inWaiting()
        return self.port.read(n), (self.port.name,self.port.baudrate)
        
    def readLast(self):
        ''' reads data until a full new message is read '''
        buffer = ''# initialize the buffer
        while True:
            buffer = buffer + self.port.read(self.port.inWaiting()).decode()# concat the buffer
            if '\n' in buffer: # check to see if there are any full messages
                lines = buffer.split('\n')# split on the line feed
                #print(lines)# uncomment to debug
                # return the most recent message
                if lines[-1].endswith('\r'):
                    return lines[-1][:-1].encode(),(self.port.name,self.port.baudrate)# received a full message last
                else:
                    return lines[-2].encode(),(self.port.name,self.port.baudrate)# return last full message
        
    def write(self,msg):
        ''' writes a message '''
        #if isinstance(msg,typing.ByteString):
        #    print('Writing %s' % msg.decode() )
        try:
            self.port.write(msg)
        except Exception as e:
            print('Error',str(e))

    def readLine(self):
        ''' reads one line of data '''
        super().readLine()
        try:
            return self.port.readline(), (self.port.name,self.port.baudrate)
        except (OSError, serial.SerialException):
            return None, None

    def isOpen(self):
        ''' Checks if the port is open'''
        if self.port != []:
            return self.port.isOpen()
        else:
            return False
        
class tcp_port(CommsPort):
    
    def __init__(self,address='127.0.0.1',port=4444,timeout=1000,connect=True,debug=False,filename=''):
        super().__init__(address+'@'+str(port))
        self.debug=debug
        self.filename=filename
        print("INIT PORT")
        if connect: self.open(address,port,timeout)
        
    def open(self,address,port,timeout=0):
        if(self.debug):
            try:
                self.file=open(self.filename, mode='rb')
                print('Done')
                return True
            except Exception as e:
                print('Error. ',str(e))
                return False
            self.set_name('%s@%d with %d s timeout'%(address,port,timeout))
            super().open()
        else:
            try:
                self.port = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                if timeout != 0: self.port.settimeout(timeout)
                self.port.connect((address,port))
                print('Done')
                return True
            except Exception as e:
                self.port = None
                print('Error. ',str(e))
                return False
        
    def send(self,message,address,port):
        ''' send a message to the port '''
        self.port.sendto(message,(address,port))
        
    def read(self,n):
        ''' reads a message of n bytes from the port '''
        if self.debug:
            try:
                time.sleep(.1)
                msg=self.file.read(n)
                return msg,None
            except Exception as e:
                print(str(e))
                return None, str(e)
        try:
            read_socket,_,_=select.select([self.port],[],[],60)
            if(len(read_socket)!=0):
                message = self.port.recv(n)
                return message,None
            else:
                return None,None
        except Exception as e:
            print(str(e))
            return None, str(e)
    def readLine(self):
        ''' reads one line of data '''
        super().readLine()
        try:
            return self.port.readline(), (self.port.name,self.port.baudrate)
        except (OSError, serial.SerialException):
            return None, None
class UDPServerPort(CommsPort):
    '''  a class for a udp server port '''
    def __init__(self,address='',port=0,timeout=1000,connect=True):
        ''' constructor '''
        super().__init__(address+'@'+str(port))
        if connect: self.open(address,port,timeout)
        
    def open(self,address,port,timeout=0):
        ''' opens the port '''
        self.set_name('%s@%d with %d s timeout'%(address,port,timeout))
        super().open()
        
        try:
            self.port = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
            self.port.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
            self.port.bind((address,port))
            if timeout != 0:
                self.port.settimeout(timeout)
                print('Done')
        except Exception as e:
            self.port = None
            print('Error. ',str(e))
        
    def send(self,message,address,port):
        ''' send a message to the port '''
        self.port.sendto(message,(address,port))
        
    def read(self,n):
        ''' reads a message of n bytes from the port '''
        message, address  = self.port.recvfrom(n)
        return message, address

class PortFinder(object):
    '''
    an abstract class for finding available ports
    '''
    def __init__(self):
        ''' constructor '''
        pass

    def update(self):
        ''' updates the list of available ports '''
        pass

class SerialPortFinder(object):
    '''
    This class is designed to find all available ports to the user.
    '''
    def __init__(self):
        ''' Constructor '''
        self.ports = []
        if sys.platform.startswith('win'):
            self.ports = ['COM%s' % (i + 1) for i in range(256)]
        elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
            # this excludes your current terminal "/dev/tty"
            self.ports = glob.glob('/dev/tty[A-Za-z]*')
        elif sys.platform.startswith('darwin'):
            self.ports = glob.glob('/dev/tty.*')
        else:
            raise EnvironmentError('Unsupported platform')
        self.update()

    def update(self):
        ''' Updates the ports available to the user '''
        self.result = []
        for port in self.ports:
            try:
                s = serial.Serial(port)
                s.close()
                self.result.append(port)
            except (OSError, serial.SerialException):
                pass
            
def testUDPServerClass():
    ''' tests the udpserver receive functionality'''
    port = UDPServerPort('',8000)
    message,address = port.read(1024)
    print(str(address),message,len(message))
    port.close()
    
def testUDPPortReader():
    port = UDPServerPort('',8112,1)
    
    i = 0
    while i < 500:
        message,address = port.read(1024)
        print(str(address),message)
        i += 1
            
    port.close()
    #reader.stopLog()
    
def testUDPPortWriter():
    print('Starting')
    port = UDPServerPort('',7654)
    
    for i in range(10):
        msg = 'message'
        print('Sending message',i,'... ',end='')
        port.send(bytes(msg,'utf-8'),'10.1.10.58',7654)
        print('Done')
        time.sleep(1)
        
    port.close()
        
    print('Done')
    
def testSerialReadLine():
    ser = SerialPort('COM8',9600,timeout=0.5)
    
    for i in range(10):
        line,addr = ser.readLine()
        print(i,addr,line.decode().strip())
    
    ser.close()

def testSerialReadLast():
    ser = SerialPort('COM8',9600,timeout=0)
    
    for i in range(10):
        line = ser.readLast()
        print(i,line.decode().strip())
    
    ser.close()

if __name__ == '__main__':
    #send0SD()
    #set1540Poll()
    testUDPServerClass()
    #testUDPPortReader()
    #testUDPPortWriter()
    #testSerialReadLast()
    #testSerialReadLine()
    
    


'''
Created on Jan 29, 2019

@author: reynolds
'''

import sys, os, struct
from wrtdk.parser.parser import parser

class hcsr04(parser):
    ''' A class for parsing the HCSR04 messages'''

    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit superclass
        
        # initialize properties
        self.reset()
        
        # define constants
        self.M_2_LSB = 1e4
        self.NS_2_LSB = 0.1
        self.LSB_2_M = 100 * 1e-6
        self.LSB_2_NS = 10
        
    def reset(self):
        ''' resets the parser properties '''
        self._ns = []
        self._alt = []
        
    def get(self,d=0,t_ns=0):
        return struct.pack('>iI',
                           int(d*self.M_2_LSB),
                           int(t_ns*self.NS_2_LSB))
        
    def parse(self,msg):
        ''' parses the wrt hcsr04 acoustic sensor message '''
        try:
            self._alt,self._ns = struct.unpack('>iI',msg)
            self._alt *= self.LSB_2_M
            self._ns *= self.LSB_2_NS
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
    
    def getData(self):
        ''' returns the data
        altitude in m
        time in nanoseconds'''
        return [self._alt,self._ns]


'''
Created on Apr 17, 2019

@author: reynolds
'''
from wrtdk.io.streamer.serial_streamer import serial_streamer
import sys
from PyQt5.QtWidgets import QApplication,QMainWindow, QPushButton, QGridLayout,QWidget
from PyQt5.QtCore import pyqtSlot
from wrtdk.comms.comms import SerialPort

class test(QMainWindow):
    '''
    classdocs
    '''

    def __init__(self):
        ''' Constructor '''
        QMainWindow.__init__(self)
        
        main = QWidget(self)
        btn = QPushButton('Connect')
        btn.clicked.connect(self._connect)
        log = QPushButton('Log')
        log.clicked.connect(self._log)
        
        l = QGridLayout()
        l.addWidget(btn)
        l.addWidget(log)
        
        main.setLayout(l)
        self.setFocus()
        self.setCentralWidget(main)
        
        self._port = None
        self._thread = None
        
    def _connect(self):
        if self._port is None:
            self._port = SerialPort('COM8',9600)
            self._thread = serial_streamer(self._port)
            self._thread.new_msg.connect(self._new_msg)
            self._thread.start()
        else:
            self._thread.stop()
            self._thread.wait()
            self._thread = None
            self._port = None
    
    def _log(self):
        if self._thread.isRunning():
            if self._thread.isLogging():
                self._thread.stopLog()
            else:
                self._thread.startLog('test.gps','wb')
        
    @pyqtSlot(list,str,int,int)
    def _new_msg(self,data,status,system,sensor):
        print(data,status,system,sensor)
        
if __name__ == '__main__':
    app = QApplication(sys.argv)
    aw = test()
    aw.show()
    app.exec_()


'''
Created on Aug 17, 2018

@author: reynolds
'''

class FileLogger(object):
    ''' Writes data to a file
    '''
    def __init__(self):
        ''' Constructor '''
        self.file = None

    def open(self,filename,ftype='w+',append=False):
        ''' Opens the file'''
        print('Opening ' + filename + ' ... ',end='')
        self.file = open(filename,ftype)
        print('Done')
        
    ''' writes a string to the file '''
    def write(self,string):
        if not self.file.closed:
            self.file.write(string)
        
    ''' Indicates whether the file is logging '''
    def isLogging(self):
        return self.file is not None
    
    ''' Closes the file '''
    def close(self):
        if self.file is None:
            return
        print('Closing ' + str(self.file.name) + ' ... ',end='')
        self.file.close()
        self.file = None
        print('Done')


'''
Created on Nov 20, 2018

@author: reynolds
'''

from OpenGL.GL import *
from OpenGL.GLU import *

from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QStatusBar, QSizePolicy, QVBoxLayout, QWidget, QApplication, QPushButton
from PyQt5.QtOpenGL import QGLWidget
import sys

class MainWindow(QWidget):

    def __init__(self):
        super(MainWindow, self).__init__()
        
        self.widget = CubeWidget(self,0,0,0,pz=.6)
        self.button = QPushButton('Change')
        self.button.clicked.connect(self.onClick)
        self.statusbar = QStatusBar()
        self.statusbar.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        layout = QVBoxLayout()
        layout.addWidget(self.widget)
        layout.addWidget(self.button)
        layout.addWidget(self.statusbar)
        layout.setContentsMargins(5, 5, 5, 5)
        self.setLayout(layout)
        
    def onClick(self):
        self.widget.update(0, 0, 180)

class OpenGLWidget(QGLWidget):

    def __init__(self, parent):
        QGLWidget.__init__(self, parent)
        self.setMinimumSize(100, 100)
        #LMB = left mouse button
        #True: fires mouseMoveEvents even when not holding down LMB
        #False: only fire mouseMoveEvents when holding down LMB
        self.setMouseTracking(False)
        self.screen = Screen()

    def initializeGL(self):
        glClearColor(0, 0, 0, 1)
        glClearDepth(1.0)
        glEnable(GL_DEPTH_TEST)

    def resizeGL(self, width, height):
        #glViewport is needed for proper resizing of QGLWidget
        aspect = float(width)/float(height)
        self.screen.update(2*aspect,2,2)
        
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(self.screen.getX0(),self.screen.getX(),
                self.screen.getY0(),self.screen.getY(),
                self.screen.getZ0(),self.screen.getZ())
        #glOrtho(0, width, 0, height, -1, 1)
        #gluPerspective(45, 1.0 * width / height, 0.1, 100.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

    def paintGL(self):
        #Renders a triangle... obvious (and deprecated!) stuff
        #w, h = self.width(), self.height()# not needed anymore
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
        
        # draw the graphic
        self.graphic()

    def graphic(self):
        pass
    
    def mousePressEvent(self, event):
        pass

    def mouseMoveEvent(self, event):
        pass

    def mouseReleaseEvent(self, event):
        pass
    
class CubeWidget(OpenGLWidget):
    
    def __init__(self,parent=None,roll=0,pitch=0,yaw=0,px=.2,py=.2,pz=.2):
        self._px = px
        self._py = py
        self._pz = pz
        OpenGLWidget.__init__(self,parent)
        self.update(roll,pitch,yaw)
        
    def update(self,roll=0,pitch=0,yaw=0):
        self._roll = roll
        self._pitch = pitch
        self._yaw = yaw
        self.glDraw()
        
    def graphic(self):
        glTranslatef(0,0,0)
        
        #string = 'Roll: %.2f, Pitch: %.2f, Yaw: %.2f' % (self._roll,self._pitch,self._yaw)
        #drawtext((-.75,-.75,-.75),string)
        
        glRotatef(self._pitch,1.0,0.0,0.0)
        glRotatef(self._yaw,0.0,-1.0,0.0)
        glRotatef(self._roll,0.0,0.0,1.0)
        
        r = ScreenCube(self.screen.getW(),self.screen.getH(),self.screen.getT(),
                       self._px,self._py,self._pz)
        
        glBegin(GL_QUADS)
        
        glColor3f(0.0,1.0,0.0)
        glVertex3f( r.getX(), r.getY(),r.getZ0())# 1.0, 0.2,-1.0)
        glVertex3f(r.getX0(), r.getY(),r.getZ0())#-1.0, 0.2,-1.0) 
        glVertex3f(r.getX0(), r.getY(), r.getZ())#-1.0, 0.2, 1.0) 
        glVertex3f( r.getX(), r.getY(), r.getZ())# 1.0, 0.2, 1.0)
        
        glColor3f(1.0,0.5,0.0)
        glVertex3f( r.getX(),r.getY0(), r.getZ())#1.0,-0.2, 1.0)
        glVertex3f(r.getX0(),r.getY0(), r.getZ())#-1.0,-0.2, 1.0)        
        glVertex3f(r.getX0(),r.getY0(),r.getZ0())#-1.0,-0.2,-1.0)        
        glVertex3f( r.getX(),r.getY0(),r.getZ0())# 1.0,-0.2,-1.0)
         
        glColor3f(1.0,0.0,0.0)        
        glVertex3f( r.getX(), r.getY(), r.getZ())# 1.0, 0.2, 1.0)
        glVertex3f(r.getX0(), r.getY(), r.getZ())#-1.0, 0.2, 1.0)        
        glVertex3f(r.getX0(),r.getY0(), r.getZ())#-1.0,-0.2, 1.0)        
        glVertex3f( r.getX(),r.getY0(), r.getZ())# 1.0,-0.2, 1.0)
        
        glColor3f(1.0,1.0,0.0)    
        glVertex3f( r.getX(),r.getY0(),r.getZ0())# 1.0,-0.2,-1.0)
        glVertex3f(r.getX0(),r.getY0(),r.getZ0())#-1.0,-0.2,-1.0)
        glVertex3f(r.getX0(), r.getY(),r.getZ0())#-1.0, 0.2,-1.0)        
        glVertex3f( r.getX(), r.getY(),r.getZ0())# 1.0, 0.2,-1.0)
        
        glColor3f(0.0,0.0,1.0)    
        glVertex3f(r.getX0(), r.getY(), r.getZ())#-1.0, 0.2, 1.0)
        glVertex3f(r.getX0(), r.getY(),r.getZ0())#-1.0, 0.2,-1.0)        
        glVertex3f(r.getX0(),r.getY0(),r.getZ0())#-1.0,-0.2,-1.0)        
        glVertex3f(r.getX0(),r.getY0(), r.getZ())#-1.0,-0.2, 1.0)
        
        glColor3f(1.0,0.0,1.0)    
        glVertex3f( r.getX(), r.getY(),r.getZ0())# 1.0, 0.2,-1.0)
        glVertex3f( r.getX(), r.getY(), r.getZ())# 1.0, 0.2, 1.0)
        glVertex3f( r.getX(),r.getY0(), r.getZ())# 1.0,-0.2, 1.0)        
        glVertex3f( r.getX(),r.getY0(),r.getZ0())# 1.0,-0.2,-1.0)
        
        glEnd()
     
class TriangeWidget(OpenGLWidget):
    
    def __init__(self,parent=None):
        OpenGLWidget.__init__(self,parent)
        
    def graphics(self):
        glBegin(GL_TRIANGLES)
        glColor3f(1, 0, 0)
        glVertex3f(self.screen.getX0(),self.screen.getY0(), 0)
        glColor3f(0, 1, 0)
        glVertex3f(0,self.screen.getY(), 0)
        glColor3f(0, 0, 1)
        glVertex3f(self.screen.getX(), 0, 0)
        glEnd()
        
    def mouseReleaseEvent(self, event):
        OpenGLWidget.mouseReleaseEvent(self, event)
        
        x, y = event.x(), event.y()
        #w, h = self.width(), self.height()
        #required to call this to force PyQt to read from the correct, updated buffer 
        #see issue noted by @BjkOcean in comments!!!
        glReadBuffer(GL_FRONT)
        data = self.grabFrameBuffer()#builtin function that calls glReadPixels internally
        data.save("test.png")
        rgba = QColor(data.pixel(x, y)).getRgb()#gets the appropriate pixel data as an RGBA tuple
        message = "You selected pixel ({0}, {1}) with an RGBA value of {2}.".format(x, y, rgba)
        statusbar = self.parent().statusbar#goes to the parent widget (main window QWidget) and gets its statusbar widget
        statusbar.showMessage(message)

class Screen(object):
    
    def __init__(self,x=1,y=1,z=1):
        self.update(x,y,z)
        
    def update(self,x=1,y=1,z=1):
        self._w, self._h, self._t = x,y,z
        self._x0,self._x = -x/2.0, x/2.0
        self._y0,self._y = -y/2.0, y/2.0
        self._z0,self._z = -z/2.0, z/2.0
        
    def getX0(self):
        return self._x0
    
    def getX(self):
        return self._x
    
    def getY0(self):
        return self._y0
    
    def getY(self):
        return self._y
    
    def getZ0(self):
        return self._z0
    
    def getZ(self):
        return self._z
    
    def getW(self):
        return self._w
    
    def getH(self):
        return self._h
    
    def getT(self):
        return self._t
    
class ScreenCube(object):
    
    def __init__(self,w=1,h=1,t=1,pw=0.1,ph=0.1,pt=0.2):
        if pw > 1: pw = 1
        if ph > 1: ph = 1
        if pt > 1: pt = 1
        self._w, self._h, self._t = w * pw, h * ph, t * pt
        self._x0, self._y0, self._z0 = -self._w/2.0, -self._h/2.0, -self._t/2.0
        self._x, self._y, self._z = -self._x0, -self._y0, -self._z0
        
    def getW(self):
        return self._w
    
    def getH(self):
        return self._h
        
    def getX0(self):
        return self._x0
    
    def getY0(self):
        return self._y0
    
    def getZ0(self):
        return self._z0
    
    def getX(self):
        return self._x
    
    def getY(self):
        return self._y
    
    def getZ(self):
        return self._z

if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MainWindow()
    window.resize(10,10)
    window.setWindowTitle("Color Picker Demo")
    window.show()
    app.exec_()




'''
Created on Aug 17, 2018

@author: reynolds
'''

import os, sys, struct, math
from wrtdk.parser.parser import parser

class pni(parser):
    ''' a class for parsing the pni vector
    magnetometer binary message '''

    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit superclass
        
        # define constants
        self.NT_2_COUNT = 75/1000
        self.COUNT_2_NT = 1000/75
        self.MSG_LEN = 12
        
        # initialize properties
        self.reset()
        
    def reset(self):
        ''' resets the properties '''
        self._bx = self._nan()
        self._by = self._nan()
        self._bz = self._nan()
        
    def get(self,bx=0,by=0,bz=0):
        ''' returns a pni vector mag message '''
        return struct.pack('>iii4s',
                    int(bx*self.NT_2_COUNT),
                    int(by*self.NT_2_COUNT),
                    int(bz*self.NT_2_COUNT),
                    b':00\x00')
        
    def parse(self,msg):
        ''' parses the pni messages '''
        self.reset()
        
        try:
            self._bx,self._by,self._bz = struct.unpack('>iii',msg[:-4])
            self._bx *= self.COUNT_2_NT
            self._by *= self.COUNT_2_NT
            self._bz *= self.COUNT_2_NT
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def _total(self):
        ''' returns the total field value '''
        return math.sqrt(self._bx ** 2 + self._by ** 2 + self._bz ** 2)
            
    def getData(self):
        ''' returns all the data 
        0) bx in n
        1) by in nT
        2) bz in nT
        3) bTotal in nT '''
        return [self._bx,self._by,self._bz,
                self._total()]